<StackLayout class="p-10">


    <GridLayout class="peek-nav-section" rows="auto" columns="auto, *, auto">
    </GridLayout>
    <StackLayout class="hr"></StackLayout>

    <StackLayout class="peek-information-section">

        <button class="btn"
                [class.bg-success]="indexStatus.cacheForOfflineEnabled"
                [text]="indexStatus.cacheForOfflineEnabled ? 'Disable Offline Support':'Enable Offline Support'"
                (tap)="toggleOfflineMode()"></button>

        <label text="Search Index Offline Mode Enabled : {{indexStatus.cacheForOfflineEnabled}}" textWrap="true"></label>
        <label text="Search Index Load Complete : {{indexStatus.initialLoadComplete}}" textWrap="true"></label>
        <label text="Search Index Load Progress : {{indexStatus.loadProgress}}/{{indexStatus.loadTotal}}" textWrap="true"></label>
        <label text="Search Index Last Check : {{indexStatus.lastCheck}}" textWrap="true"></label>

        <label text="Search Object Offline Mode Enabled : {{objectStatus.cacheForOfflineEnabled}}" textWrap="true"></label>
        <label text="Search Object Load Complete : {{objectStatus.initialLoadComplete}}" textWrap="true"></label>
        <label text="Search Object Load Progress : {{objectStatus.loadProgress}}/{{objectStatus.loadTotal}}" textWrap="true"></label>
        <label text="Search Object Last Check : {{objectStatus.lastCheck}}" textWrap="true"></label>

    </StackLayout>
</StackLayout>