"""added SI objectId index

Peek Plugin Database Migration Script

Revision ID: 2c6cad1f280e
Revises: 30f0d0f925ca
Create Date: 2018-07-01 18:03:05.530637

"""

# revision identifiers, used by Alembic.
revision = '2c6cad1f280e'
down_revision = '30f0d0f925ca'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index('idx_SearchIndex_objectId', 'SearchIndex', ['objectId'], unique=False, schema='core_search')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index('idx_SearchIndex_objectId', table_name='SearchIndex', schema='core_search')
    # ### end Alembic commands ###