import {Component, OnInit} from "@angular/core";

@Component({
    selector: 'search-admin',
    templateUrl: 'search.component.html'
})
export class SearchComponent  implements OnInit {

    ngOnInit() {

    }
}
